from pathlib import Path

from pydantic import BaseModel, Field, field_validator


class Software(BaseModel):
    name: str
    owner: str
    repo: str
    version: str
    folder: str
    filename_format: str = Field(..., alias="filenameFormat")

    @field_validator("folder")
    @classmethod
    def replace_tilde(cls, value: str):
        return value.replace("~", Path.home().as_posix())

    @property
    def filename(self) -> str:
        return self.filename_format.replace("{{version}}", self.version)

    @property
    def absolute_path(self) -> Path:
        return Path(self.folder, self.filename)


class GithubSoftware(Software):
    base_url: str = "https://api.github.com"

    @property
    def release_url(self):
        if self.version == "latest":
            release_path = self.version
        else:
            release_path = f"tags/{self.version}"

        return f"{self.base_url}/repos/{self.owner}/{self.repo}/releases/{release_path}"


class GitlabSoftware(Software):
    base_url: str = Field(..., alias="baseUrl")

    @property
    def release_url(self):
        return f"{self.base_url}/{self.owner}/{self.repo}/-/releases/{self.version}"


class Asset(BaseModel):
    name: str
    url: str
    content: bytes | str | None = None
